﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Windows.Forms;
using System.Speech.Synthesis;
using System.IO;
using System.Collections.ObjectModel;

namespace Event
{
    // Basierend auf dem Artikel
    // https://dotnet-snippets.de/snippet/text-to-speech-sprachausgabe/1392

    public partial class Form1 : Form
    {
        private static SpeechSynthesizer speaker;
        private static bool vorlesen = false;
        // private List<VoiceInfo> voiceinfos;
        private ReadOnlyCollection<InstalledVoice> voiceinfos;

        // Demo-Texte für die bei mir installierten Sprachen
        private string german = "Das ist ein Demo-Text für den Microsoft Speechsynthesizer.\r\n" +
                                "Heute ist ein schöner Tag zum programmieren einer kleinen Demo " +
                                "für die Verwendung von Text-to-Speech.";
        private string english = "This is a demo text for the Microsoft Speech Synthesizer.\r\n" +
                                 "Today is a nice day to program a little Demo for " +
                                 "using text-to-speech.";
        private string french = "Ceci est un texte de démonstration pour le synthétiseur vocal Microsoft.\r\n" +
                                "Aujourd'hui est une belle journée pour programmer un peu ' " +
                                "Démo pour l'utilisation de la synthèse vocale.";
        private string italian = "Questo è un testo dimostrativo per Microsoft Speech Synthesizer.\r\n" +
                                 "Oggi è una bella giornata per programmare un po '" +
                                 "Demo per l'utilizzo della sintesi vocale.";
        private string spanish = "Este es un texto de demostración para Microsoft Speech Synthesizer.\r\n" +
                                 "Hoy es un buen día para programar un poco Demostración " +
                                 "para usar texto a voz.";
        private string chinese = "Zhè shì Microsoft yǔyīn héchéng qì de yǎnshì wénběn.\r\n" +
                                 "Jīntiān shì biānchéng de hǎo rìzi shǐyòng wénzì zhuǎn yǔyīn de yǎnshì.";

        public Form1()
        {
            InitializeComponent();

            // SpeechSynthesizer-Objekt erstellen
            speaker = new SpeechSynthesizer();
            speaker.SetOutputToDefaultAudioDevice();
            speaker.Rate = 0;
            speaker.Volume = 100;
            // Eventhandler für Event angeben
            speaker.SpeakCompleted += Speaker_SpeakCompleted;


            voiceinfos = speaker.GetInstalledVoices();
            foreach (InstalledVoice iv in voiceinfos)
            {
                VoiceInfo vi = iv.VoiceInfo;
                comboBoxVocals.Items.Add(vi.Name);
            }

            if (comboBoxVocals.Items.Count > 0)
            {
                comboBoxVocals.SelectedItem = comboBoxVocals.Items[0];
            }
            else
            {
                buttonVorlesen.Enabled = false;
                MessageBox.Show("Keine Stimmen vorhanden!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void Speaker_SpeakCompleted(object sender, SpeakCompletedEventArgs e)
        {
            // Eventhandler für das Event SpeakCompleted vom SpeechSynthesizer-Objekt
            vorlesen = false;
            buttonVorlesen.Text = "Vorlesen";
        }

        private void buttonVorlesen_Click(object sender, EventArgs e)
        {
            // Starten das Vorlesen oder bricht ab; je nach Zustand
            if (!vorlesen)
            {
                speaker.SpeakAsync(textBoxSpeechText.Text);
                buttonVorlesen.Text = "Abbruch";
            }
            else
            {
                speaker.SpeakAsyncCancelAll();
                buttonVorlesen.Text = "Vorlesen";
            }
            vorlesen = !vorlesen;
        }

        private void numericUpDownSpeechrate_ValueChanged(object sender, EventArgs e)
        {
            // Sprechgeschwindigkeit setzen; wirkt erst bei neuem Vorlesen
            speaker.Rate = (int)numericUpDownSpeechrate.Value;
        }

        private void numericUpDownVolume_ValueChanged(object sender, EventArgs e)
        {
            // Lautstärke setzen; wirkt erst bei neuem Vorlesen
            speaker.Volume = (int)numericUpDownVolume.Value;
        }

        private void radioButtonPause_Click(object sender, EventArgs e)
        {
            // Umschalten zwischen Vorlesen und Pausieren
            radioButtonPause.Checked = !radioButtonPause.Checked;

            if (radioButtonPause.Checked)
            {
                radioButtonPause.Text = "Vorlesen";
                speaker.Pause();
            }
            else
            {
                radioButtonPause.Text = "Pausieren";
                speaker.Resume();
            }
        }

        // Bei Auswahl einer neuen Stimme
        private void comboBoxVocals_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Momentanes Vorlesen sofort beenden
            if (speaker.State == SynthesizerState.Speaking)
                speaker.SpeakAsyncCancelAll();

            // Infos zur gewählten Stimme anzeigen
            int index = comboBoxVocals.SelectedIndex;
            comboBoxVocals.SelectedItem = comboBoxVocals.Items[index];
            speaker.SelectVoice(comboBoxVocals.SelectedItem.ToString());
            textBoxVocalDescription.Text = voiceinfos[index].VoiceInfo.Name + "\r\n" +
                                           voiceinfos[index].VoiceInfo.Culture + "\r\n" +
                                           voiceinfos[index].VoiceInfo.Gender.ToString() + "\r\n" +
                                           voiceinfos[index].VoiceInfo.Age.ToString();

            // Vorzulesenden Text für die Sprache laden
            switch (voiceinfos[index].VoiceInfo.Culture.ToString().Substring(0, 2))
            {
                case "de":
                    textBoxSpeechText.Text = german;
                    break;
                case "en":
                    textBoxSpeechText.Text = english;
                    break;
                case "fr":
                    textBoxSpeechText.Text = french;
                    break;
                case "es":
                    textBoxSpeechText.Text = spanish;
                    break;
                case "it":
                    textBoxSpeechText.Text = italian;
                    break;
                case "zh":
                    textBoxSpeechText.Text = chinese;
                    break;
                default:
                    textBoxSpeechText.Text = "";
                    break;
            }

        }

        private void buttonFileSelect_Click(object sender, EventArgs e)
        {
            // Öffnet und liest den Inhalt einer Textdatei
            // in die TextBox. Zeichensatz sollte möglichst UTF-8 sein

            string filePath = string.Empty;

            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.InitialDirectory = @".\";
                openFileDialog.Filter = "txt files (*.txt)|*.txt";

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    //Get the path of specified file
                    filePath = openFileDialog.FileName;

                    //Read the contents of the file into a stream
                    Stream fileStream = openFileDialog.OpenFile();

                    using (StreamReader reader = new StreamReader(fileStream, true))
                    {
                        textBoxSpeechText.Text = reader.ReadToEnd();
                    }
                }
            }
        }

        // Spricht einen Text, wenn der Mauszeiger kurz übder dem
        // Steuerelement verweilt
        private void textBoxSpeechText_MouseHover(object sender, EventArgs e)
        {
            // speaker.Speak("Enthält den vorzulesenden Text");
        }

        private void buttonFileSelect_MouseHover(object sender, EventArgs e)
        {
            // speaker.Speak("Lädt eine vorzulesende Datei");
        }

        private void comboBoxVocals_MouseHover(object sender, EventArgs e)
        {
            // speaker.Speak("Wählt die Stimme");
        }

        private void textBoxVocalDescription_MouseHover(object sender, EventArgs e)
        {
            // speaker.Speak("Zeigt Infos zur Stimme");
        }

        private void radioButtonPause_MouseHover(object sender, EventArgs e)
        {
            // speaker.Speak("Pausiert das Vorlesen oder setzt es fort");
        }

        private void buttonVorlesen_MouseHover(object sender, EventArgs e)
        {
            // speaker.Speak("Liest den Text in der Textbox vor");
        }
    }
}
